'use strict';
var fs = require('fs');

module.export /* jshint node: true */

function usingProxy() {
  var usingProxyArg = !!process.argv.filter(function (arg) {
    return arg.indexOf('--proxy') === 0 || arg.indexOf('-pr') === 0 || arg.indexOf('-pxy') === 0;
  }).length;

  var hasGeneratedProxies = false;
  var proxiesDir = process.env.PWD + '/server/proxies';
  try {
    fs.lstatSync(proxiesDir);
    hasGeneratedProxies = true;
  } catch (e) {}

  return usingProxyArg || hasGeneratedProxies;
}

function usingJhipster() {
  let isJhipster = !!process.argv.filter(function (arg) {
    return arg.indexOf('--jhipster') === 0 || arg.indexOf('-jhi') === 0
  }).length;
  return isJhipster;
}

module.exports = function(environment) {
  var ENV = {
    modulePrefix: 'ember-cellar',
    environment: environment,
    rootURL: '/',
    locationType: 'auto',
    EmberENV: {
      FEATURES: {
        // Here you can enable experimental features on an ember canary build
        // e.g. 'with-controller': true
      },
      EXTEND_PROTOTYPES: {
        // Prevent Ember Data from overriding Date.parse.
        Date: false
      }
    },

    APP: {
      // Here you can pass flags/options to your application instance
      // when it is created
      proxy: usingProxy(), // todo set proxy according to the --proxy option passed to ember cli
      jhipster: usingJhipster()
    }
  };

  if (environment === 'development') {
    // ENV.APP.LOG_RESOLVER = true;
    // ENV.APP.LOG_ACTIVE_GENERATION = true;
    // ENV.APP.LOG_TRANSITIONS = true;
    // ENV.APP.LOG_TRANSITIONS_INTERNAL = true;
    // ENV.APP.LOG_VIEW_LOOKUPS = true;
  }

  if (environment === 'test') {
    // Testem prefers this...
    ENV.locationType = 'none';

    // keep test console output quieter
    ENV.APP.LOG_ACTIVE_GENERATION = false;
    ENV.APP.LOG_VIEW_LOOKUPS = false;

    ENV.APP.rootElement = '#ember-testing';
  }

  if (environment === 'production') {

  }
  ENV['ember-simple-auth'] = {
    authorizer: 'authorizer:token'
  };
  ENV['ember-simple-auth-token'] = {
    refreshAccessTokens: true,
    refreshLeeway: 300, // Refresh the token 5 minutes (300s) before it expires.
    serverTokenEndpoint: '/api/authenticate',
    identificationField: 'username',
    passwordField: 'password',
    tokenPropertyName: 'id_token'
  };
  return ENV;
};
