package fr.ippon.jhipster.cellar.domain;

import com.fasterxml.jackson.annotation.JsonIgnore;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import javax.persistence.*;
import java.io.Serializable;
import java.time.LocalDate;
import java.util.HashSet;
import java.util.Set;
import java.util.Objects;

/**
 * A Rack.
 */
@Entity
@Table(name = "rack")
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Rack implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @Column(name = "name")
    private String name;

    @Column(name = "nb_columns")
    private Integer nbColumns;

    @Column(name = "nb_rows")
    private Integer nbRows;

    @Column(name = "image")
    private String image;

    @Column(name = "created_at")
    private LocalDate createdAt;

    @OneToMany(mappedBy = "rack")
    @JsonIgnore
    @Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
    private Set<Bottle> bottles = new HashSet<>();

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public Rack name(String name) {
        this.name = name;
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Integer getNbColumns() {
        return nbColumns;
    }

    public Rack nbColumns(Integer nbColumns) {
        this.nbColumns = nbColumns;
        return this;
    }

    public void setNbColumns(Integer nbColumns) {
        this.nbColumns = nbColumns;
    }

    public Integer getNbRows() {
        return nbRows;
    }

    public Rack nbRows(Integer nbRows) {
        this.nbRows = nbRows;
        return this;
    }

    public void setNbRows(Integer nbRows) {
        this.nbRows = nbRows;
    }

    public String getImage() {
        return image;
    }

    public Rack image(String image) {
        this.image = image;
        return this;
    }

    public void setImage(String image) {
        this.image = image;
    }

    public LocalDate getCreatedAt() {
        return createdAt;
    }

    public Rack createdAt(LocalDate createdAt) {
        this.createdAt = createdAt;
        return this;
    }

    public void setCreatedAt(LocalDate createdAt) {
        this.createdAt = createdAt;
    }

    public Set<Bottle> getBottles() {
        return bottles;
    }

    public Rack bottles(Set<Bottle> bottles) {
        this.bottles = bottles;
        return this;
    }

    public Rack addBottles(Bottle bottle) {
        this.bottles.add(bottle);
        bottle.setRack(this);
        return this;
    }

    public Rack removeBottles(Bottle bottle) {
        this.bottles.remove(bottle);
        bottle.setRack(null);
        return this;
    }

    public void setBottles(Set<Bottle> bottles) {
        this.bottles = bottles;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        Rack rack = (Rack) o;
        if (rack.getId() == null || getId() == null) {
            return false;
        }
        return Objects.equals(getId(), rack.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(getId());
    }

    @Override
    public String toString() {
        return "Rack{" +
            "id=" + getId() +
            ", name='" + getName() + "'" +
            ", nbColumns='" + getNbColumns() + "'" +
            ", nbRows='" + getNbRows() + "'" +
            ", image='" + getImage() + "'" +
            ", createdAt='" + getCreatedAt() + "'" +
            "}";
    }
}
