import { Component, OnInit, OnDestroy } from '@angular/core';
import { Response } from '@angular/http';
import { ActivatedRoute, Router } from '@angular/router';
import { Subscription } from 'rxjs/Rx';
import { EventManager, ParseLinks, PaginationUtil, AlertService } from 'ng-jhipster';

import { Bottle } from './bottle.model';
import { BottleService } from './bottle.service';
import { ITEMS_PER_PAGE, Principal } from '../../shared';
import { PaginationConfig } from '../../blocks/config/uib-pagination.config';

@Component({
    selector: 'jhi-bottle',
    templateUrl: './bottle.component.html'
})
export class BottleComponent implements OnInit, OnDestroy {
bottles: Bottle[];
    currentAccount: any;
    eventSubscriber: Subscription;

    constructor(
        private bottleService: BottleService,
        private alertService: AlertService,
        private eventManager: EventManager,
        private principal: Principal
    ) {
    }

    loadAll() {
        this.bottleService.query().subscribe(
            (res: Response) => {
                this.bottles = res.json();
            },
            (res: Response) => this.onError(res.json())
        );
    }
    ngOnInit() {
        this.loadAll();
        this.principal.identity().then((account) => {
            this.currentAccount = account;
        });
        this.registerChangeInBottles();
    }

    ngOnDestroy() {
        this.eventManager.destroy(this.eventSubscriber);
    }

    trackId(index: number, item: Bottle) {
        return item.id;
    }
    registerChangeInBottles() {
        this.eventSubscriber = this.eventManager.subscribe('bottleListModification', (response) => this.loadAll());
    }

    private onError(error) {
        this.alertService.error(error.message, null, null);
    }
}
