import { Injectable } from '@angular/core';
import { Http, Response, URLSearchParams, BaseRequestOptions } from '@angular/http';
import { Observable } from 'rxjs/Rx';

import { Rack } from './rack.model';
import { DateUtils } from 'ng-jhipster';

@Injectable()
export class RackService {

    private resourceUrl = 'api/racks';

    constructor(private http: Http, private dateUtils: DateUtils) { }

    create(rack: Rack): Observable<Rack> {
        const copy = this.convert(rack);
        return this.http.post(this.resourceUrl, copy).map((res: Response) => {
            return res.json();
        });
    }

    update(rack: Rack): Observable<Rack> {
        const copy = this.convert(rack);
        return this.http.put(this.resourceUrl, copy).map((res: Response) => {
            return res.json();
        });
    }

    find(id: number): Observable<Rack> {
        return this.http.get(`${this.resourceUrl}/${id}`).map((res: Response) => {
            const jsonResponse = res.json();
            jsonResponse.createdAt = this.dateUtils
                .convertLocalDateFromServer(jsonResponse.createdAt);
            return jsonResponse;
        });
    }

    query(req?: any): Observable<Response> {
        const options = this.createRequestOption(req);
        return this.http.get(this.resourceUrl, options)
            .map((res: Response) => this.convertResponse(res))
        ;
    }

    delete(id: number): Observable<Response> {
        return this.http.delete(`${this.resourceUrl}/${id}`);
    }

    private convertResponse(res: Response): Response {
        const jsonResponse = res.json();
        for (let i = 0; i < jsonResponse.length; i++) {
            jsonResponse[i].createdAt = this.dateUtils
                .convertLocalDateFromServer(jsonResponse[i].createdAt);
        }
        res.json().data = jsonResponse;
        return res;
    }

    private createRequestOption(req?: any): BaseRequestOptions {
        const options: BaseRequestOptions = new BaseRequestOptions();
        if (req) {
            const params: URLSearchParams = new URLSearchParams();
            params.set('page', req.page);
            params.set('size', req.size);
            if (req.sort) {
                params.paramsMap.set('sort', req.sort);
            }
            params.set('query', req.query);

            options.search = params;
        }
        return options;
    }

    private convert(rack: Rack): Rack {
        const copy: Rack = Object.assign({}, rack);
        copy.createdAt = this.dateUtils
            .convertLocalDateToServer(rack.createdAt);
        return copy;
    }
}
