package fr.ippon.jhipster.cellar.web.rest;

import fr.ippon.jhipster.cellar.JhipstercellarApp;

import fr.ippon.jhipster.cellar.domain.Bottle;
import fr.ippon.jhipster.cellar.repository.BottleRepository;
import fr.ippon.jhipster.cellar.web.rest.errors.ExceptionTranslator;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.web.PageableHandlerMethodArgumentResolver;
import org.springframework.http.MediaType;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

/**
 * Test class for the BottleResource REST controller.
 *
 * @see BottleResource
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = JhipstercellarApp.class)
public class BottleResourceIntTest {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final LocalDate DEFAULT_CREATED_AT = LocalDate.ofEpochDay(0L);
    private static final LocalDate UPDATED_CREATED_AT = LocalDate.now(ZoneId.systemDefault());

    private static final Integer DEFAULT_YROW = 1;
    private static final Integer UPDATED_YROW = 2;

    private static final Integer DEFAULT_XCOLUMN = 1;
    private static final Integer UPDATED_XCOLUMN = 2;

    private static final Boolean DEFAULT_FLIPPED = false;
    private static final Boolean UPDATED_FLIPPED = true;

    @Autowired
    private BottleRepository bottleRepository;

    @Autowired
    private MappingJackson2HttpMessageConverter jacksonMessageConverter;

    @Autowired
    private PageableHandlerMethodArgumentResolver pageableArgumentResolver;

    @Autowired
    private ExceptionTranslator exceptionTranslator;

    @Autowired
    private EntityManager em;

    private MockMvc restBottleMockMvc;

    private Bottle bottle;

    @Before
    public void setup() {
        MockitoAnnotations.initMocks(this);
        BottleResource bottleResource = new BottleResource(bottleRepository);
        this.restBottleMockMvc = MockMvcBuilders.standaloneSetup(bottleResource)
            .setCustomArgumentResolvers(pageableArgumentResolver)
            .setControllerAdvice(exceptionTranslator)
            .setMessageConverters(jacksonMessageConverter).build();
    }

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Bottle createEntity(EntityManager em) {
        Bottle bottle = new Bottle()
            .name(DEFAULT_NAME)
            .createdAt(DEFAULT_CREATED_AT)
            .yrow(DEFAULT_YROW)
            .xcolumn(DEFAULT_XCOLUMN)
            .flipped(DEFAULT_FLIPPED);
        return bottle;
    }

    @Before
    public void initTest() {
        bottle = createEntity(em);
    }

    @Test
    @Transactional
    public void createBottle() throws Exception {
        int databaseSizeBeforeCreate = bottleRepository.findAll().size();

        // Create the Bottle
        restBottleMockMvc.perform(post("/api/bottles")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(bottle)))
            .andExpect(status().isCreated());

        // Validate the Bottle in the database
        List<Bottle> bottleList = bottleRepository.findAll();
        assertThat(bottleList).hasSize(databaseSizeBeforeCreate + 1);
        Bottle testBottle = bottleList.get(bottleList.size() - 1);
        assertThat(testBottle.getName()).isEqualTo(DEFAULT_NAME);
        assertThat(testBottle.getCreatedAt()).isEqualTo(DEFAULT_CREATED_AT);
        assertThat(testBottle.getYrow()).isEqualTo(DEFAULT_YROW);
        assertThat(testBottle.getXcolumn()).isEqualTo(DEFAULT_XCOLUMN);
        assertThat(testBottle.isFlipped()).isEqualTo(DEFAULT_FLIPPED);
    }

    @Test
    @Transactional
    public void createBottleWithExistingId() throws Exception {
        int databaseSizeBeforeCreate = bottleRepository.findAll().size();

        // Create the Bottle with an existing ID
        bottle.setId(1L);

        // An entity with an existing ID cannot be created, so this API call must fail
        restBottleMockMvc.perform(post("/api/bottles")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(bottle)))
            .andExpect(status().isBadRequest());

        // Validate the Alice in the database
        List<Bottle> bottleList = bottleRepository.findAll();
        assertThat(bottleList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    public void getAllBottles() throws Exception {
        // Initialize the database
        bottleRepository.saveAndFlush(bottle);

        // Get all the bottleList
        restBottleMockMvc.perform(get("/api/bottles?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_UTF8_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(bottle.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME.toString())))
            .andExpect(jsonPath("$.[*].createdAt").value(hasItem(DEFAULT_CREATED_AT.toString())))
            .andExpect(jsonPath("$.[*].yrow").value(hasItem(DEFAULT_YROW)))
            .andExpect(jsonPath("$.[*].xcolumn").value(hasItem(DEFAULT_XCOLUMN)))
            .andExpect(jsonPath("$.[*].flipped").value(hasItem(DEFAULT_FLIPPED.booleanValue())));
    }

    @Test
    @Transactional
    public void getBottle() throws Exception {
        // Initialize the database
        bottleRepository.saveAndFlush(bottle);

        // Get the bottle
        restBottleMockMvc.perform(get("/api/bottles/{id}", bottle.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_UTF8_VALUE))
            .andExpect(jsonPath("$.id").value(bottle.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME.toString()))
            .andExpect(jsonPath("$.createdAt").value(DEFAULT_CREATED_AT.toString()))
            .andExpect(jsonPath("$.yrow").value(DEFAULT_YROW))
            .andExpect(jsonPath("$.xcolumn").value(DEFAULT_XCOLUMN))
            .andExpect(jsonPath("$.flipped").value(DEFAULT_FLIPPED.booleanValue()));
    }

    @Test
    @Transactional
    public void getNonExistingBottle() throws Exception {
        // Get the bottle
        restBottleMockMvc.perform(get("/api/bottles/{id}", Long.MAX_VALUE))
            .andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    public void updateBottle() throws Exception {
        // Initialize the database
        bottleRepository.saveAndFlush(bottle);
        int databaseSizeBeforeUpdate = bottleRepository.findAll().size();

        // Update the bottle
        Bottle updatedBottle = bottleRepository.findOne(bottle.getId());
        updatedBottle
            .name(UPDATED_NAME)
            .createdAt(UPDATED_CREATED_AT)
            .yrow(UPDATED_YROW)
            .xcolumn(UPDATED_XCOLUMN)
            .flipped(UPDATED_FLIPPED);

        restBottleMockMvc.perform(put("/api/bottles")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(updatedBottle)))
            .andExpect(status().isOk());

        // Validate the Bottle in the database
        List<Bottle> bottleList = bottleRepository.findAll();
        assertThat(bottleList).hasSize(databaseSizeBeforeUpdate);
        Bottle testBottle = bottleList.get(bottleList.size() - 1);
        assertThat(testBottle.getName()).isEqualTo(UPDATED_NAME);
        assertThat(testBottle.getCreatedAt()).isEqualTo(UPDATED_CREATED_AT);
        assertThat(testBottle.getYrow()).isEqualTo(UPDATED_YROW);
        assertThat(testBottle.getXcolumn()).isEqualTo(UPDATED_XCOLUMN);
        assertThat(testBottle.isFlipped()).isEqualTo(UPDATED_FLIPPED);
    }

    @Test
    @Transactional
    public void updateNonExistingBottle() throws Exception {
        int databaseSizeBeforeUpdate = bottleRepository.findAll().size();

        // Create the Bottle

        // If the entity doesn't have an ID, it will be created instead of just being updated
        restBottleMockMvc.perform(put("/api/bottles")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(bottle)))
            .andExpect(status().isCreated());

        // Validate the Bottle in the database
        List<Bottle> bottleList = bottleRepository.findAll();
        assertThat(bottleList).hasSize(databaseSizeBeforeUpdate + 1);
    }

    @Test
    @Transactional
    public void deleteBottle() throws Exception {
        // Initialize the database
        bottleRepository.saveAndFlush(bottle);
        int databaseSizeBeforeDelete = bottleRepository.findAll().size();

        // Get the bottle
        restBottleMockMvc.perform(delete("/api/bottles/{id}", bottle.getId())
            .accept(TestUtil.APPLICATION_JSON_UTF8))
            .andExpect(status().isOk());

        // Validate the database is empty
        List<Bottle> bottleList = bottleRepository.findAll();
        assertThat(bottleList).hasSize(databaseSizeBeforeDelete - 1);
    }

    @Test
    @Transactional
    public void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(Bottle.class);
        Bottle bottle1 = new Bottle();
        bottle1.setId(1L);
        Bottle bottle2 = new Bottle();
        bottle2.setId(bottle1.getId());
        assertThat(bottle1).isEqualTo(bottle2);
        bottle2.setId(2L);
        assertThat(bottle1).isNotEqualTo(bottle2);
        bottle1.setId(null);
        assertThat(bottle1).isNotEqualTo(bottle2);
    }
}
